require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'
require 'csv'

module StudioGame
	class Game
		attr_reader :title
		def initialize(title)
			@title = title
			@players = []
		end

		def load_players(filename)
			CSV.foreach(filename) do |row|
				player = Player.new(row[0], row[1].to_i)
				add_player(player)
			end
		end

		def high_score_etnry(player)
			formatted_name = player.name.ljust(20,'.')
			 "#{formatted_name } #{player.score}"
		end

		def save_high_scores(filname = "high_scores.txt")
			File.open(filname, "w") do |file|
				file.puts "#{@title} High Score"
				@players.sort.each do |player|
					file.puts high_score_etnry(player)
				end
			end
		end


		def add_player(player)
			@players << player 
		end

		def play(rounds)
			puts @players
			treasures = TreasureTrove::TREASURES 
			puts "\nThere are #{treasures.size} treasures to be found:"
			treasures.each do |treasure|
				puts "A #{treasure.name} is worth #{treasure.points} points"
			end
			1.upto(rounds) do |round|
				if block_given?
					break if yield
				end
				puts "\nRound #{round}:"
				@players.each do |player|
					GameTurn.take_turn(player)
					puts player
				end
			end
		end

		def print_name_and_health(player)
			puts "#{player.name} (#{player.health})"
		end

		def total_points
			@players.reduce(0) { |sum, player| sum + player.points }
		end

		def print_stats
			@players.sort.each do |player|
				puts "\n#{player.name}'s point totals:"
				player.each_found_treasure do |treasure|
					puts "#{treasure.points} total #{treasure.name} points"
				end
				puts "#{player.points} grand total points"
			end

			strong, wimpy = @players.partition { |player| player.strong? }
			puts "\n#{title} Statistics:"
			puts "\n#{strong.size} strong players:"
			strong.each do |player|
				print_name_and_health(player)
			end
			puts "\n#{wimpy.size} wimpy players:"
			wimpy.each do |player|
				print_name_and_health(player)
			end
			puts "\n#{title} High Scores:"
			@players.sort.each do |player|
				puts high_score_etnry(player)
			end
			puts "\nTotal treasure points found: #{total_points}"
		end
	end
end

if __FILE__ == $0
	player1 = Player.new('moe')
	player2 = Player.new('larry', 60)
	knuckelheads = Game.new("Knuckleheads")
	knuckelheads.add_player(player1)
	knuckelheads.add_player(player2)
	knuckelheads.play(5)
	knuckelheads.print_stats
	knuckelheads.play(5) do
		knuckelheads.total_points >= 2000
	end
end